/*  main.c
 * 
 * Copyright(C) 2000-2005 Mikael Berthe <bmikael@lists.lilotux.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Creation: 2000-12-17
 * $LastChangedRevision$
 * $LastChangedDate$
 * Last modification: 2001-03-24 18:06
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include <signal.h>

#include <errno.h>

#include "getopt.h"

#include "main.h"
#include "reseau.h"
#include "edition.h"

//char *VERSION   = "0.09";
char *DATE      = "$Date: 2001/03/29 07:10:42 $";


char File[1024];
int Dbg;
struct TCoord Contact;

FILE *DFLog = NULL;


// Initialisation (valeurs par dfaut)
void Init(void)
{
  Dbg   = 0;
  memset(&Contact, 0, sizeof(Contact));
  File[0] = 0;

#ifdef HAVE_LIBPTH
  pth_init();
#endif
}

// Affichage des messages d'erreur / de dbogage
void DBGprintf(int niveau, const char *format, ...)
{
  int quitter = 0;
  va_list args;
  char tmpmsg[512];

  // Si le niveau est ngatif, il s'agit d'une erreur critique
  if (niveau < 0) {
      niveau *= -1;
      quitter = niveau;
  }

  if ((quitter) || (Dbg & niveau) || (!niveau)) {
      memset(tmpmsg, 0, sizeof(tmpmsg));
      
      va_start(args, format);
      vsnprintf(tmpmsg, sizeof(tmpmsg), format, args);
      va_end(args);

      if (TerminalInitialise)
          AffichageDebogage(tmpmsg);
      else
          fprintf(stderr, "%s\n", tmpmsg);

      if (DFLog)
          fprintf(DFLog, "%s", tmpmsg);
  }

  if (quitter) {
      if (DFLog)
          fclose(DFLog);
      TermInit(1);
      fprintf(stderr, "%s\n", tmpmsg);
      exit(quitter);
  }
}

// Charge le fichier en mmoire
// Retourne 0 en cas de succs
int ChargementFichier(char fichier[])
{
  FILE *df;
  char buf[256];
  int  t;

  df = fopen(fichier, "r");
  if (df == NULL)
      return(1);
  buf[255] = 0;

  while(1) {
      fgets(buf, 255, df);
      if (feof(df))
          break;
      t = strlen(buf);
      while (t && ((buf[t-1] == '\x0d') || (buf[t-1] == '\x0a'))) {
          buf[--t] = 0;
      }
      InsertionLigne(-1, buf, 1, 0);
  }
  fclose(df);

  // On vite d'avoir une liste vide
  if (!Contenu)
      InsertionLigne(0, "", 1, 0);

  return(0);
}

// Enregistre le fichier
// Retourne 0 en cas de succs
int EnregistrementFichier(char *fichier)
{
  FILE *df;
  struct TLigne *tmpl;

  // On retire tous les verous (pour tre sr d'avoir diffus toutes les
  // lignes)
  RetraitVerrous();

  if (DemandeJetonSauvegarde()) {
      DBGprintf(0, "Impossible d'obtenir le jeton de sauvegarde ;\n"
              "fichier non enregistr.\n");
      return(-1);
  }

  df = fopen(fichier, "w");
  if (df == NULL)
      return(1);

  POURCHAQUELIGNE(tmpl) {
      fwrite(tmpl->ligne, 1, strlen(tmpl->ligne), df);
      // XXX Pas de vrification
      fwrite("\x0a", 1, 1, df);
  }
  fclose(df);
  return(0);
}

// Affichage de la version (et rvision CVS si elle existe)
void Version(void)
{
  char tmpver[16];
  int have_libpth = 0;

  printf("VID\tversion %s", VERSION);
  if (strlen(DATE) > 18)
  {
      strncpy(tmpver, &DATE[7], 10);
      tmpver[10] = 0;
      printf("  %s", tmpver);
  }

#ifdef HAVE_LIBPTH
  have_libpth = 1;
#endif

  printf("\n(%cpth)", ((have_libpth) ? '+' : '-'));
  printf("\n\n");
}

// Affichage de l'aide
void Usage(char *nom)
{
  printf("Usage: %s [OPTIONS] fichier\n", nom);
  printf("Options :\n");
  printf("  --help           -h   " "affiche ce message, puis quitte\n");
  printf("  --verbose=VALEUR -V   "
         "niveau de verbosit (0-15)      [%d]\n", Dbg);
  printf("                        0 : Pas de dbogage\n");
  printf("                        1 : Dbogage initialisation\n");
  printf("                        2 : Dbogage rseau\n");
  printf("                        4 : Dbogage diteur\n");
  printf("                        8 : Dbogage avanc\n");
  printf("  --version        -v   " "affiche la version, puis quitte\n");
  printf("  --port           -p   " "port  utiliser\n");
  printf("  --editeur        -e   " "adresse d'un autre diteur (contact)\n");
  printf("fichier :\n");
  printf(" nom du fichier  diter\n\n");
  // printf("Report bugs to <XXX>.\n");  :-)
}

// On prend en compte l'option. Renvoie FAUX s'il y a une erreur.
int UseOption(int option, char *arg)
{
  char *c;

  switch(option)
  {
    case 'V':
      Dbg = atoi(arg);
      break;
    case 'p':
      Hote.port = atoi(arg);
      break;
    case 'l':
      {
          char logname[32];
          sprintf(logname, "/tmp/vid_%d.log", getpid());
          DFLog = fopen(logname, "w");
          if (DFLog)
              DBGprintf(1, "Log dans le fichier <<%s>>\n", logname);
      }
      break;
    case 'e':
      {
          char nom[64];

          strncpy(nom, arg, 63);
          nom[63] = 0;

          c = strchr(nom, ':');
          if (c)
              *c = '\0';
          ConversionAdresse(Contact.IP, nom);
          c = strchr(arg, ':');
          if (c)
              Contact.port = atoi(c+1);
          else
              Contact.port = PORT;
      }
      break;
    case 's':
      HauteurDebug = atoi(arg);
      if (HauteurDebug < 1)
          HauteurDebug = 1;
      break;
    default:
      return(0);
  }

  return(1);
}

void Interruption(int val)
{
    DBGprintf(-255, "Rception de SIGINT (%d)\n", val);
    exit(-1); // Normalement, cette ligne est inutile
}


int main(int argc, char **argv)
{
  struct sigaction * act;

  act = malloc(sizeof(struct sigaction));
  act->sa_handler = Interruption;

  // Pour intercepter les interruptions en cas de problme
  signal(SIGINT, Interruption);

  Init();

  // Initialisation de la partie rseau
  InitReseau();

  // tude des options :
  {
      int opt;

      static struct option long_options[] = {
          {"port", 1, 0, 'p'},
          {"editeur", 1, 0, 'e'},
          {"verbose", 1, 0, 'V'},
          {"version", 0, 0, 'v'},
          {"log", 0, 0, 'l'},
          {"sizedebug", 1, 0, 's'},
          {"help", 0, 0, 'h'},
          {0, 0, 0, 0}
      };

      while ((opt = getopt_long(argc, argv,
                      "p:e:V:s:vhl", long_options, NULL)) != EOF)
      {
          if (opt == 'h') {       // --help
              Usage(argv[0]);
              exit(1);
          }
          if (opt == 'v') {       // --version
              Version();
              exit(0);
          }

          if (!UseOption(opt, optarg)) {
              printf("Tapez `%s --help' pour plus d'informations\n", argv[0]);
              exit(-1);
          }
      }
      // Nom du fichier prcis ?
      if (argc == optind+1) {
          File[255]=0;
          if (argv[optind][0] == '/')
              strncpy(File, argv[optind], 255);
          else {
              strncpy(File, getenv("PWD"), 255);
              strcat(File, "/");
              strncat(File, argv[optind], 255 - strlen(File));
          }
      } else if (!Contact.IP[0]) {
          DBGprintf(DBG_INIT, "ERREUR : Nom du fichier  diter manquant.\n");
          Usage(argv[0]);
          exit(2);
      }
  }

  DBGprintf(DBG_INIT, "Fichier : [%s]\n", File);
  DBGprintf(DBG_INIT, "Port    : %d\n", Hote.port);

  if (Contact.IP[0]) {
      DBGprintf(DBG_INIT, "Contact IP   : %s\n", Contact.IP);
      DBGprintf(DBG_INIT, "Contact Port : %d\n", Contact.port);
      if (File[0]) {
          DBGprintf(0, "Nom de fichier inutile ; NON PRIS EN COMPTE\n");
          File[0] = 0;  // (Inutile...)
      }
  } else {
      if (ChargementFichier(File)) {
          DBGprintf(-DBG_INIT, "Impossible de charger le fichier.\n");
      }
      // On a le jeton global
      JetonGlobal = 1;
      // On a le jeton de sauvegarde
      JetonSauvegarde = 1;
  }

  CreationSocketServeur(Hote.port);
  // Dbut de l'dition
  TermInit(0);

  // On avertit le contact de notre arrive
  if (Contact.IP[0]) {
      struct TMessage tmpmsg;
      char ipport[80];

      sprintf(ipport, ":%d", Hote.port);
      tmpmsg.donnees = (unsigned char*) ipport;
      tmpmsg.longueur = strlen((char*)tmpmsg.donnees)+1;
      tmpmsg.msg_id = MSG_DEB_SESSION;
      if (EnvoiMessageEditeur(&Contact, &tmpmsg, 1) == -1)
          DBGprintf(-DBG_RESEAU, "Impossible de joindre le contact !\n");
      if (TransfertFichier(&Contact)) {
          DBGprintf(-(DBG_INIT+DBG_RESEAU),
                  "Impossible de transfrer le fichier du contact.\n");
      }
  }

  Edition();

  // TODO Librer la mmoire...
  TermInit(1);
  DestructionSocketServeur();
  if (DFLog)
      fclose(DFLog);
  printf("\nBye !\n");
#ifdef HAVE_LIBPTH
  pth_kill();
#endif
  exit(0);
}

/* vim: set expandtab cindent cinoptions=>4^-2\:2:  For the ViMasters... */
