/*  reseau.c
 * 
 * Copyright(C) 2000-2005 Mikael Berthe <bmikael@lists.lilotux.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Creation: MiKael, 2000-12-17
 * $LastChangedRevision$
 * $LastChangedDate$
 * Last modification: 2001-03-24 16:00
 */

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>

#include "reseau.h"
#include "main.h"
#include "edition.h"

#define TOUS  NULL

// Coordonnes de cet diteur
struct TCoord Hote;
// Coordonnes des diteurs (qui traitent le mme fichier)
struct TCoord *CEditeurs;

// Contenu du fichier
struct TLigne *Contenu;
struct TLigne *Fin;

// Jeton et verrou globaux
int JetonGlobal;
int VerrouGlobal;
// Jeton pour sauvegarder le fichier
int JetonSauvegarde;

char *DBGMSG[] = {
    "",
    "DEB_SESSION",
    "SESSION",
    "FIN_SESSION",
    "HERITAGE",
    "CHARGEMENT",
    "RESERVATION",
    "DIFFUSION",
    "DEM_JETON",
    "DON_JETON",
    "NEGATIF",
    "AJOUT",
    "SUPPRESSION",
    "LIBERATION",
    "OK"
};


// Initialisation de la partie rseau
void InitReseau(void)
{
  memset(&Hote, 0, sizeof(Hote));
  // (IP non ncessaire)
  Hote.port = PORT;
  CEditeurs = NULL;
  Fin = Contenu = NULL;
  JetonSauvegarde = 0;
  JetonGlobal = 0;
  VerrouGlobal = 0;
}

void AjouterEditeur(char IP[], int port)
{
  struct TCoord *tmpc;

  if (CEditeurs == NULL) {
      tmpc = CEditeurs = (struct TCoord*) malloc(sizeof(struct TCoord));
  } else {
      for (tmpc = CEditeurs ; tmpc->suiv ; tmpc = tmpc->suiv)
          if ((tmpc->port == port) && (!strcmp(tmpc->IP, IP)))
              return;   // diteur dj rfrenc

      tmpc->suiv = (struct TCoord*) malloc(sizeof(struct TCoord));
      tmpc = tmpc->suiv;
  }

  DBGprintf(DBG_RESEAU, "   AJOUTEREDITEUR (%s:%d)\n", IP, port);
  memset(tmpc, 0, sizeof(*tmpc));
  strncpy(tmpc->IP, IP, 15);
  tmpc->port = port;
}

void SupprimerEditeur(char IP[], int port)
{
  struct TCoord *tmpc;

  if (!CEditeurs)
      return;

  // Suppression du premier de la liste
  if ((!strncmp(CEditeurs->IP, IP, 15)) && (CEditeurs->port == port)) {
      tmpc = CEditeurs;
      CEditeurs = CEditeurs->suiv;
      free(tmpc);
      return;
  }

  // Suppression d'un autre maillon
  for (tmpc = CEditeurs ;  ; tmpc = tmpc->suiv)
  {
      if (!tmpc->suiv)
          break;
      if ((!strncmp(tmpc->suiv->IP, IP, 15)) && (tmpc->suiv->port == port)) {
          struct TCoord *tmpc2 = tmpc->suiv;
          tmpc->suiv = tmpc2->suiv;
          free(tmpc2);
          break;
      }
  }
}

// Gestion des sessions ( la rception d'un message)
void GestionSessions(struct TMessage *msg, char *IP_exp)
{
  char IP[32];
  int port;
  char ipport[80];
  int editeurConnu = 0;
  struct TCoord *tmpc;
  struct TMessage tmpmsg;

  if (msg->donnees[0] == ':') {
      strcpy(IP, IP_exp);
      port = atoi((char *)msg->donnees + 1);
  } else {
      char *c;
      strcpy(IP, (char*) msg->donnees);
      c = strchr(IP, ':');
      *c = '\0';
      port = atoi(c+1);
  }

  POURCHAQUEEDITEUR(tmpc) {
      if ((!strcmp(tmpc->IP, IP)) && (tmpc->port == port)) {
          editeurConnu = 1;
          break;
      }
  }

  if (msg->msg_id == MSG_FIN_SESSION) {
      DBGprintf(DBG_RESEAU, "R: MSG_FIN_SESSION (%s).\n", IP_exp);
      SupprimerEditeur(IP_exp, port);
      // (+ si pas connu : pb !)
      return;
  }

  // Reste la gestion des dbuts de session.
  // Si l'diteur est dj dans notre liste, c'est fini
  // Idem s'il n'y a pas de destinataires pour la diffusion
  if (editeurConnu)
      return;

  if (msg->msg_id == MSG_DEB_SESSION) {
      struct TCoord tmpcEditeur;

      sprintf(ipport, "%s:%d", IP, port);
      // Diffusion de l'arrive du nouvel diteur
      POURCHAQUEEDITEUR(tmpc) {
          tmpmsg.donnees = (unsigned char*) ipport;
          tmpmsg.longueur = strlen((char*)tmpmsg.donnees)+1;
          tmpmsg.msg_id = MSG_DEB_SESSION;
          EnvoiMessageEditeur(tmpc, &tmpmsg, 1);
      }

      // Rponse au nouvel diteur
      sprintf(ipport, ":%d", Hote.port);
      strcpy(tmpcEditeur.IP, IP);
      tmpcEditeur.port = port;
      tmpmsg.donnees = (unsigned char*) ipport;
      tmpmsg.longueur = strlen((char*)tmpmsg.donnees)+1;
      tmpmsg.msg_id = MSG_SESSION;
      EnvoiMessageEditeur(&tmpcEditeur, &tmpmsg, 1);
  }

  // Ajout de l'diteur dans notre liste
  AjouterEditeur(IP, port);

  return;
}

// Envoi d'un message  un diteur.
// Si fermerDS est nul, on ne ferme pas la socket et on la retourne
// Si fermerDS vaut 1, on attend un ACK et on ferme la socket
// Retour :
// fermerDS = 1 :  renvoie -1 en cas d'erreur, 0 sinon
// fermerDS = 0 :  renvoie -1 en cas d'erreur, le ds sinon
int EnvoiMessageEditeur(struct TCoord *editeur, struct TMessage *msg,
        int fermerDS)
{
  int sc;
#ifdef HAVE_LIBPTH
  pth_event_t ev;
#endif
  int ack;

  sc = ConnexionClient(editeur->IP, editeur->port);
  if (sc < 0) {
      DBGprintf(0, "ERR : Connexion  %s:%d impossible.\n",
              editeur->IP, editeur->port);
      return(-1);
  }

  DBGprintf(DBG_RESEAU, "T: MSG  (%s:%d) mid=%d [%s]\n",
          editeur->IP, editeur->port, msg->msg_id, DBGMSG[msg->msg_id]);
  EmissionMessage(sc, msg);
  
  if (fermerDS) {
      // Les  nouveaux  diteurs ne veulent pas d'ack !  :-(
      if ((msg->msg_id != MSG_DEB_SESSION) && (msg->msg_id != MSG_SESSION)) {
          //DBGprintf(DBG_RESEAU, "Attente d'ACK\n");
#ifdef HAVE_LIBPTH
          ev = pth_event(PTH_EVENT_FD | PTH_UNTIL_FD_READABLE, sc);
          pth_wait(ev);
#endif
          read(sc, &ack, sizeof(ack));
          // XXX : vrifier que ack == MSG_OK.
          //DBGprintf(DBG_RESEAU, "ACK reu.\n");
      }
      close(sc);
      return(0);
  }
  return(sc);
}

// Envoi d'un message  tous les diteurs connus
void DiffusionMessage(struct TMessage *msg)
{
  struct TCoord *i;

  POURCHAQUEEDITEUR(i) {
      EnvoiMessageEditeur(i, msg, 1);
  }
}

// Diffusion d'un message (concernant une ligne)
// Si dest == TOUS (NULL), on envoie  tous les diteurs connus
void DiffusionMessageLigne(struct TCoord *dest, int type, struct TLigne *ligne)
{
  struct TMessage mymsg;
  int lignenum;

  lignenum = NumeroDeLigne(ligne);

  if (type == MSG_DIFFUSION) {
      // On intgre la ligne dans le message
      mymsg.donnees = (unsigned char*)
                      malloc(sizeof(lignenum)+strlen(ligne->ligne)+1);
      memcpy(mymsg.donnees, (void*)&lignenum, sizeof(lignenum));
      strcpy((char*) mymsg.donnees+sizeof(lignenum), ligne->ligne);
      mymsg.longueur = sizeof(lignenum)+strlen(ligne->ligne)+1;
  } else {
      mymsg.donnees = (unsigned char*) malloc(sizeof(lignenum));
      memcpy(mymsg.donnees, (void*)&lignenum, sizeof(lignenum));
      mymsg.longueur = sizeof(lignenum);
  }
  mymsg.msg_id = type;

  if (dest)
    EnvoiMessageEditeur(dest, &mymsg, 1);
  else
    DiffusionMessage(&mymsg);

  free(mymsg.donnees);
}

// ===== ===== Gestion des jetons et verrous ===== =====

// L'diteur demande (par rseau) le jeton pour une ligne
// Retourne 0 en cas de succs
int DemandeJeton(struct TLigne *ligne)
{
  struct TCoord *i;
  int sc;
  struct TMessage mymsg;
  unsigned char rep;
  int lignenum;
#ifdef HAVE_LIBPTH
  pth_event_t ev;
#endif

  if (ligne->jeton)
      return(0);

  lignenum = NumeroDeLigne(ligne);
  mymsg.donnees = (unsigned char*) malloc(sizeof(lignenum));
  memcpy(mymsg.donnees, (void*)&lignenum, sizeof(lignenum));
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_DEM_JETON;

  POURCHAQUEEDITEUR(i) {
      if (VerrouGlobal) {
          DBGprintf(DBG_VERB, ".  VG=1 => On ne demande pas de Jeton...\n");
          return 1;
      }

      sc = EnvoiMessageEditeur(i, &mymsg, 0);
      if (sc >= 0) {
#ifdef HAVE_LIBPTH
          ev = pth_event(PTH_EVENT_FD | PTH_UNTIL_FD_READABLE, sc);
          pth_wait(ev);
#endif
          read(sc, &rep, sizeof(rep));
          close(sc);
          DBGprintf(DBG_RESEAU, "T: DEM_JETON ligne %d  (%s:%d)... "
                  "Rponse reue : %s.\n", lignenum, i->IP, i->port,
                  DBGMSG[rep]);
          DBGprintf(DBG_VERB, "   (Ligne %p)\n", ligne);
          if (rep == MSG_DON_JETON) {
              ligne->jeton = 1;
              break;
          }
      }
  }

  free(mymsg.donnees);
  return(!ligne->jeton);
}

// L'diteur veut verrouiller une ligne
// Retourne 0 en cas de succs
int DemandeVerrou(struct TLigne *ligne)
{
  if ((ligne->jeton) && (ligne->verrou))
      return(0);

  if (DemandeJeton(ligne)) {
      // On suppose que quelqu'un a verrouill la ligne...
      ligne->verrou = 1;
      return(1);
  }

  // Le VG a pu tre pos entre temps !
  if (VerrouGlobal) {
      DBGprintf(DBG_VERB, ".  VG=1 => On ne vrouille pas...\n");
      return(1);
  }

  // On dtient le jeton
  if (!ligne->verrou) {
      ligne->verrou = 1;
      DiffusionMessageLigne(TOUS, MSG_RESERVATION, ligne);
  }
  return(0);
}

// L'diteur libre la ligne
// Retourne 0 en cas de succs
int FinVerrou(struct TLigne *ligne)
{
  if (ligne->jeton) {
      if (ligne->verrou) {
          if (VerrouGlobal && (!JetonGlobal))
              return(1);    // Impossible !
          DiffusionMessageLigne(TOUS, MSG_DIFFUSION, ligne);
      }
      ligne->verrou = 0;
  }
  return(0);
}

// Rception (rseau) d'une demande de jeton
void ReceptionDemandeJeton(struct TLigne *ligne, int ds)
{
  unsigned char rep;

  if ((ligne->jeton) && (!ligne->verrou)) {
      rep = MSG_DON_JETON;
      ligne->jeton = 0;
  }
  else
      rep = MSG_NEGATIF;

  DBGprintf(DBG_RESEAU, "T: JETON -> %s\n", DBGMSG[rep]);
  DBGprintf(DBG_VERB, "   (Ligne %p)\n", ligne);
  write(ds, &rep, sizeof(rep));
}

// Rception (rseau) d'une demande de JS
void ReceptionDemandeJetonSauvegarde(int ds)
{
  unsigned char rep;

  if (JetonSauvegarde) {
      rep = MSG_DON_JETON;
      JetonSauvegarde = 0;
  }
  else
      rep = MSG_NEGATIF;

  DBGprintf(DBG_RESEAU, "T: JS -> %s\n", DBGMSG[rep]);
  write(ds, &rep, sizeof(rep));
}

// L'diteur demande (par rseau) le JS
// Retourne 0 en cas de succs
int DemandeJetonSauvegarde(void)
{
  struct TCoord *i;
  int sc;
  struct TMessage mymsg;
  unsigned char rep;
  int lignenum;
#ifdef HAVE_LIBPTH
  pth_event_t ev;
#endif

  if (JetonSauvegarde)
      return(0);

  if (VerrouGlobal) {
      DBGprintf(DBG_VERB, ".  VG=1 => On ne demande pas le JS...\n");
      return 1;
  }

  lignenum = -1;    // Convention pour le JS
  mymsg.donnees = (unsigned char*) malloc(sizeof(lignenum));
  memcpy(mymsg.donnees, (void*)&lignenum, sizeof(lignenum));
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_DEM_JETON;

  // XXX On pourrait refaire une boucle autour de celle-ci
  //     pour faire plusieurs essais (+ gestion du cas de panne...)
  POURCHAQUEEDITEUR(i) {
      sc = EnvoiMessageEditeur(i, &mymsg, 0);
      if (sc >= 0) {
#ifdef HAVE_LIBPTH
          ev = pth_event(PTH_EVENT_FD | PTH_UNTIL_FD_READABLE, sc);
          pth_wait(ev);
#endif
          read(sc, &rep, sizeof(rep));
          close(sc);
          DBGprintf(DBG_RESEAU, "T: DEM_JETON Sauvegarde  (%s:%d)... "
                  "Rponse reue : %s.\n", i->IP, i->port, DBGMSG[rep]);
          if (rep == MSG_DON_JETON) {
              JetonSauvegarde = 1;
              break;
          }
      }
  }

  free(mymsg.donnees);
  return(!JetonSauvegarde);
}

// Rception (rseau) d'une demande de JG
void ReceptionDemandeJetonGlobal(int ds)
{
  unsigned char rep;

  if ((JetonGlobal) && (!VerrouGlobal)) {
      rep = MSG_DON_JETON;
      JetonGlobal = 0;
  }
  else
      rep = MSG_NEGATIF;

  DBGprintf(DBG_RESEAU, "T: DEM_JETON Global -> %s.\n", DBGMSG[rep]);
  write(ds, &rep, sizeof(rep));
}

// L'diteur demande (par rseau) le JG
// Retourne 0 en cas de succs
int DemandeJetonGlobal(void)
{
  struct TCoord *i;
  int sc;
  struct TMessage mymsg;
  unsigned char rep;
  int lignenum;
#ifdef HAVE_LIBPTH
  pth_event_t ev;
#endif

  if (JetonGlobal)
      return 0;

  if (VerrouGlobal) {
      DBGprintf(DBG_VERB, ".  VG=1 => On ne demande pas le JG...\n");
      return 1;
  }

  lignenum = 0;    // Convention pour le JG
  mymsg.donnees = (unsigned char*) malloc(sizeof(lignenum));
  memcpy(mymsg.donnees, (void*)&lignenum, sizeof(lignenum));
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_DEM_JETON;

  // XXX On pourrait refaire une boucle autour de celle-ci
  //     pour faire plusieurs essais (+ gestion du cas de panne...)
  POURCHAQUEEDITEUR(i) {
      sc = EnvoiMessageEditeur(i, &mymsg, 0);
      if (sc >= 0) {
#ifdef HAVE_LIBPTH
          ev = pth_event(PTH_EVENT_FD | PTH_UNTIL_FD_READABLE, sc);
          pth_wait(ev);
#endif
          read(sc, &rep, sizeof(rep));
          close(sc);
          DBGprintf(DBG_RESEAU, "T: DEM_JETON Global  (%s:%d)... "
                  "Rponse reue : %s.\n", i->IP, i->port, DBGMSG[rep]);
          if (rep == MSG_DON_JETON) {
              JetonGlobal = 1;
              break;
          }
      }
  }

  free(mymsg.donnees);
  return(!JetonGlobal);
}

// Met le verrou global
// Retourne 0 en cas de succs
int PoseVerrouGlobal(void)
{
  if (VerrouGlobal) {
      DBGprintf(DBG_VERB, ".  VG=1 => On ne peut pas le reposer...\n");
      return 1;
  }

  if (DemandeJetonGlobal()) {
      // On suppose que quelqu'un a pos le VG...
      VerrouGlobal = 1;
      return(1);
  }

  // On dtient le JG
  // On r-met dans tous les cas (pour les cas d'insertion d'un
  // nouvel diteur)
  {
      struct TMessage mymsg;
      int lignenum;

      VerrouGlobal = 1;

      lignenum = 0; // Convention pour le VG
      mymsg.donnees = (void *) &lignenum;
      mymsg.longueur = sizeof(lignenum);
      mymsg.msg_id = MSG_RESERVATION;
      DiffusionMessage(&mymsg);
  }

  return(0);
}

// Retire le verrou global
void RetraitVerrouGlobal(void)
{
  struct TMessage mymsg;
  int lignenum;

  if ((!JetonGlobal) || (!VerrouGlobal))
      return;

  // On a le VG
  VerrouGlobal = 0;
  lignenum = 0; // Convention pour le VG

  mymsg.donnees = (void *) &lignenum;
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_LIBERATION;
  DiffusionMessage(&mymsg);

  return;
}

// ===== ===== Fin de Gestion des jetons et verrous ===== =====

// Dverrouillage
void RetraitVerrous(void)
{
  struct TLigne *tmpl;

  POURCHAQUELIGNE(tmpl) {
      if (tmpl->jeton)
          while (FinVerrou(tmpl)) {
              DBGprintf(DBG_VERB, "Dlai d'attente retrait du VG...");
              usleep(10000);
          }
  }
}

// Fermeture de l'diteur
void FinSession(void)
{
  char ipport[16];
  struct TCoord *tmpc;
  struct TMessage msg;
  int nbjetons = 0;

  sprintf(ipport, ":%d", Hote.port);

  while (PoseVerrouGlobal()) {
      DBGprintf(DBG_VERB, "Dlai d'attente retrait du VG...");
      usleep(20000);
  }

  // S'il y a d'autres diteurs, on libre les jetons/verrous
  if (CEditeurs) {
      struct TLigne *tmpl;
      struct TMessage tmpmsg;

      // Diffusion de la fin de session
      POURCHAQUEEDITEUR(tmpc) {
          tmpmsg.donnees = (unsigned char*) ipport;
          tmpmsg.longueur = strlen((char*)tmpmsg.donnees)+1;
          tmpmsg.msg_id = MSG_FIN_SESSION;
          EnvoiMessageEditeur(tmpc, &tmpmsg, 1);
      }

      // On rend tous les jetons
      // Premire passe : on dverrouille et on compte le nombre de jetons
      POURCHAQUELIGNE(tmpl) {
          if (tmpl->jeton) {
              nbjetons++;
              // ventuellement, on retire le verrou
              FinVerrou(tmpl);
          }
      }
      // (En thorie, on a le JG...)
      if (JetonGlobal)
          nbjetons++;
      if (JetonSauvegarde)
          nbjetons++;
      // Deuxime passe : cration du message d'hritage du jeton
      if (nbjetons) {
          int i = 0;
          int *j;

          msg.longueur = nbjetons*sizeof(int);
          msg.donnees = (unsigned char*) malloc(msg.longueur);
          msg.msg_id = MSG_HERITAGE;
          j = (int*) msg.donnees;

          if (JetonGlobal)
              j[i++] = 0;   //  0 <=> JetonGlobal
          if (JetonSauvegarde)
              j[i++] = -1;  // -1 <=> JetonSauvegarde

          POURCHAQUELIGNE(tmpl) {
              if (tmpl->jeton) {
                  j[i++] = NumeroDeLigne(tmpl);
                  if (i == nbjetons)
                      break;
              }
          }
          EnvoiMessageEditeur(CEditeurs, &msg, 1);
          free(msg.donnees);
      }
  }
}

// Rception des jetons lors de la fermeture d'un diteur
void HeritageJetons(struct TMessage *msg)
{
  int i;
  int *j = (int*) msg->donnees;
  int retirerVG = 0;

  for (i=0 ; i<(msg->longueur/sizeof(int)) ; i++) {
      if (j[i]>0) {
          LigneDeNumero(j[i])->jeton = 1;
      } else if (j[i] == 0) {
          JetonGlobal = 1;
          retirerVG = 1;
      } else if (j[i] == -1) {
          JetonSauvegarde = 1;
      }
  }

  if (retirerVG) {
      // On retire le VG que l'diteur a pos avant de quitter
      VerrouGlobal = 1;
      RetraitVerrouGlobal();
  }
}

void GestionLignes(struct TMessage *msg, int socketserveur)
{
  int lignenum;
  struct TLigne *ligne;
  int ack = MSG_OK;

  memcpy((void*)&lignenum, msg->donnees, sizeof(lignenum));

  if (lignenum > 0)
      DBGprintf(DBG_RESEAU, "   Message concernant la ligne %d\n", lignenum);
  else if (lignenum == 0)
      DBGprintf(DBG_RESEAU, "   Message concernant le JG ou VG\n");
  else
      DBGprintf(DBG_RESEAU, "   Message concernant le JS\n");

  // Cas spciaux
  if (lignenum == -1) {         // On demande le JS
      ReceptionDemandeJetonSauvegarde(socketserveur);
      return;
  } else if (lignenum == 0) {  // On demande le JG
      // Soit on demande le jeton, soit on l'a et on pose le verrou
      if (msg->msg_id == MSG_DEM_JETON)
          ReceptionDemandeJetonGlobal(socketserveur);
      else {
          VerrouGlobal = 1;
          write(socketserveur, &ack, sizeof(ack));  // ACK
      }
      return;
  }

  ligne = LigneDeNumero(lignenum);
  if (!ligne) {
      DBGprintf(0, "*** Ligne %d inexistante ! ***\n", lignenum);
      return;
  }

  switch(msg->msg_id) {
    case MSG_DEM_JETON:
        ReceptionDemandeJeton(ligne, socketserveur);
        break;
    case MSG_RESERVATION:
        ligne->verrou = 1;
        write(socketserveur, &ack, sizeof(ack));    // ACK
        break;
    case MSG_DIFFUSION:
        ligne->verrou = 0;
        free(ligne->ligne);
        ligne->ligne = strdup((char*)msg->donnees + sizeof(lignenum));
        // On prvient l'diteur...
        NotificationDeModification(ligne);
        write(socketserveur, &ack, sizeof(ack));    // ACK
        break;
  }
}

void ModificationGlobale(int action, int *position)
{
  if (action == MSG_AJOUT) {
      DBGprintf(DBG_RESEAU, "   INSERTION ligne %d\n", *position);
      InsertionLigne(*position, "", 0, 1);
  } else if (action == MSG_SUPPRESSION) {
      DBGprintf(DBG_RESEAU, "   SUPPRESSION ligne %d\n", *position);
      SuppressionLigne(*position);
      // On devrait tester le rsultat...
  }
}

void EmissionFichier(int ds)
{
  struct TLigne *tmpl;
  int tailleligne;

  DBGprintf(DBG_RESEAU, "mission du contenu du fichier...\n");
  // mission du nom du fichier
  tailleligne = strlen(File);
  write(ds, &tailleligne, sizeof(tailleligne));
  write(ds, File, tailleligne);
  // Contenu
  POURCHAQUELIGNE(tmpl) {
      tailleligne = strlen(tmpl->ligne);
      write(ds, &tailleligne, sizeof(tailleligne));
      write(ds, tmpl->ligne, tailleligne);
  }
  tailleligne = -1;
  write(ds, &tailleligne, sizeof(tailleligne));
}

// Rception d'un message client (ouverture de la socket ncessaire)
int ReceptionNouvelleSocket(void)
{
  char clientIP[32];
  int nss;
  struct TMessage mymsg;
  int ack = MSG_OK;

  memset(clientIP, 0, sizeof(clientIP));
  nss = SocketClient(Socket, clientIP);
  if (nss < 0) {
      DBGprintf(DBG_RESEAU, "*** Connexion au serveur impossible (%s).\n",
              strerror(errno));
      return(0);
  }

  // Traitement du message
  ReceptionMessage(nss, &mymsg);
  DBGprintf(DBG_RESEAU, "R: (%15s)    MID : %2d  [%s]\n", clientIP,
          mymsg.msg_id, DBGMSG[mymsg.msg_id]);

  switch(mymsg.msg_id) {
    case MSG_DEB_SESSION:
    case MSG_SESSION:
    case MSG_FIN_SESSION:
        if (mymsg.msg_id == MSG_FIN_SESSION)
            write(nss, &ack, sizeof(ack));   // ACK
        GestionSessions(&mymsg, clientIP);
        break;
    case MSG_DEM_JETON:
    case MSG_RESERVATION:
    case MSG_DIFFUSION:
        GestionLignes(&mymsg, nss);
        break;
    case MSG_CHARGEMENT:
        EmissionFichier(nss);
        break;
    case MSG_HERITAGE:
        HeritageJetons(&mymsg);
        break;
    case MSG_AJOUT:
    case MSG_SUPPRESSION:
        ModificationGlobale(mymsg.msg_id, (int*) mymsg.donnees);
        write(nss, &ack, sizeof(ack));   // ACK
        break;
    case MSG_LIBERATION:
        VerrouGlobal = 0;
        write(nss, &ack, sizeof(ack));   // ACK
        break;
    default:
        DBGprintf(DBG_RESEAU, "*** MID non reconnu/implment\n");
  }

  free(mymsg.donnees);

  shutdown(nss, 2);     //close(nss);
  return(1);    // Le retour concernera l'diteur...
}

// Charge le fichier en mmoire  partir d'un diteur distant (contact)
// Retourne 0 en cas de succs
int TransfertFichier(struct TCoord *contact)
{
  struct TMessage mymsg;
  int ds;
  char buf[256];
  int lu,reste;

  mymsg.donnees = NULL;
  mymsg.longueur = 0;
  mymsg.msg_id = MSG_CHARGEMENT;

  DBGprintf(DBG_RESEAU, "Demande du transfert du contenu du fichier...\n");
  ds = EnvoiMessageEditeur(contact, &mymsg, 0);

  if (ds >= 0) {
      int tailleligne;

      DBGprintf(DBG_RESEAU, "Rception du fichier... ");
      usleep(350000);
        memset(buf, 0, sizeof(buf));
        read(ds, &tailleligne, sizeof(tailleligne));
        read(ds, buf, tailleligne);
        strncpy(File, buf, 256);  // 256 < 1024...
        DBGprintf(0, "(%s)...\n", File);
      usleep(200000);
      for (;;) {
          memset(buf, 0, sizeof(buf));
          read(ds, &tailleligne, sizeof(tailleligne));
          if (tailleligne < 0)
              break;
          if (tailleligne > 255)
              tailleligne = 255;
          reste = tailleligne;

          do {
              lu = read(ds, &buf[tailleligne-reste], reste);
              reste -= lu;
              if (reste)
                  usleep(250000);   // On attend l'arrive des donnes
          } while (reste);

          InsertionLigne(-1, buf, 0, 0);
      }
      close(ds);
  }

  return(0);
}

// Tentative d'insertion d'une ligne aprs la ligne *pos
// (pos = NULL pour insrer en tte de fichier)
// Retourne 0 en cas de succs
int Reseau_InsertionLigne(struct TLigne *pos)
{
  struct TMessage mymsg;
  int lignenum;

  if (PoseVerrouGlobal())
      return(1);

  if (pos)
      lignenum = NumeroDeLigne(pos);
  else
      lignenum = 0; //  Insertion en dbut de fichier

  mymsg.donnees = (void *) &lignenum;
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_AJOUT;

  DiffusionMessage(&mymsg);
  InsertionLigne(lignenum, "", 1, 1);

  RetraitVerrouGlobal();

  return(0);
}

// Tentative de suppression de la ligne *pos
// Retourne 0 en cas de succs
int Reseau_SuppressionLigne(struct TLigne *pos)
{
  struct TMessage mymsg;
  int lignenum;

  if (PoseVerrouGlobal())
      return(1);

  lignenum = NumeroDeLigne(pos);
  DBGprintf(DBG_DBG, ".  Reseau_SuppLigne %d (%p)\n", lignenum, pos);

  mymsg.donnees = (void *) &lignenum;
  mymsg.longueur = sizeof(lignenum);
  mymsg.msg_id = MSG_SUPPRESSION;

  DiffusionMessage(&mymsg);
  SuppressionLigne(lignenum);
  // On devrait tester le rsultat...

  RetraitVerrouGlobal();

  return(0);
}

/* vim: set expandtab cindent cinoptions=>4^-2\:2:  For the ViMasters... */
