/*  reseau.h
 * 
 * Copyright(C) 2000-2005 Mikael Berthe <bmikael@lists.lilotux.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Creation: MiKael, 2000-12-17
 * $LastChangedRevision$
 * $LastChangedDate$
 * Last modification: 2001-01-24 20:58
 */


#ifndef _RESEAU_H_
#define _RESEAU_H_

#include "reseau_socket.h"      // Pour TMessage

#define PORT        5217

#define POURCHAQUEEDITEUR(e)  for ((e) = CEditeurs ; (e) ; (e) = (e)->suiv)
#define POURCHAQUELIGNE(e)    for ((e) = Contenu ; (e) ; (e) = (e)->suiv)

// Coordonnes d'un diteur
struct TCoord {
    // char nom[32];
    char IP[16];
    int  port;
    struct TCoord *suiv;
};

// Types de messages
enum {
    MSG_DEB_SESSION = 1,    // 1  Dbut d'dition d'un fichier  (+ [IP:]port)
    MSG_SESSION,            // 2  Infos session existante       (+ port)
    MSG_FIN_SESSION,        // 3  Fin d'dition du fichier      (+ port)
    MSG_HERITAGE,           // 4  Hritage jetons en fin de session (+ liste)
    MSG_CHARGEMENT,         // 5  Transfert du fichier pour chargement
    MSG_RESERVATION,        // 6  Verrouillage d'une ligne    (+ num.)
    MSG_DIFFUSION,          // 7  Diffusion d'une ligne       (+ num + ligne)
    MSG_DEM_JETON,          // 8  Demande du jeton            (+ num)
    MSG_DON_JETON,          // 9  Transmission du jeton       (+ num)
    MSG_NEGATIF,            // 10 Rponse ngative
    MSG_AJOUT,              // 11 Ajout d'une ligne           (+ num)
    MSG_SUPPRESSION,        // 12 Suppression d'une ligne     (+ num)
    MSG_LIBERATION,         // 13 Libration d'une ligne      (+ num)
    MSG_OK                  // 14 Accus rception
};

struct TLigne {
    int  jeton;
    int  verrou;
    char *ligne;
    struct TLigne *prec;
    struct TLigne *suiv;
};


// Variables globales Rseau
extern struct TCoord Hote;          // Coordonnes de cet diteur
extern struct TCoord *CEditeurs;    // Coordonnes des autres diteurs
extern struct TLigne *Contenu;      // Pointeur sur le dbut du fichier
extern struct TLigne *Fin;          // Pointeur sur la fin du fichier
extern int    JetonSauvegarde;      // Jeton utilis lors des sauvegardes
extern int    JetonGlobal;          // Jeton utilis pour les ajouts/suppr.
extern int    VerrouGlobal;         // Verrou utilis pour les ajouts/suppr.

// Fonctions
void InitReseau(void);
void AjouterEditeur(char IP[], int port);
void SupprimerEditeur(char IP[], int port);
void FinSession(void);
int  ReceptionNouvelleSocket(void);
int  TransfertFichier(struct TCoord *);
int  EnvoiMessageEditeur(struct TCoord *, struct TMessage *, int);
int  DemandeJeton(struct TLigne *ligne);
// Et pour l'dition
int  DemandeVerrou(struct TLigne *ligne);
int  FinVerrou(struct TLigne *ligne);
void RetraitVerrous(void);
int  DemandeJetonSauvegarde(void);
int  Reseau_InsertionLigne(struct TLigne *);
int  Reseau_SuppressionLigne(struct TLigne *);

#endif  /* _RESEAU_H_ */
/* vim: set expandtab cindent cinoptions=>4^-2\:2:  For the ViMasters... */
